<?php
/**
 * Plugin Name: WP to Kintone
 * Plugin URI:  https://kintone.cybozu.com/
 * Description: Easy to create an input form which is linked to the kintone, it can be used as query forms and questionnaire form.
 * Version:	 1.3.1
 * Author:	  cybozu-kintone
 * Author URI:  cybozu-kintone
 * License:	 GPLv2
 * Text Domain: kintone
 * Domain Path: /languages
 */

/**
 * Copyright (c) 2014 Takayuki Miyauchi ( https://wpist.me/ )
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2 or, at
 * your discretion, any later version, as published by the Free
 * Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */



define( 'KINTONE_URL',  plugins_url( '', __FILE__ ) );
define( 'KINTONE_PATH', dirname( __FILE__ ) );

require_once( dirname( __FILE__ ).'/includes/class-kintone-options.php' );
require_once( dirname( __FILE__ ).'/includes/class-kintone-api.php' );
require_once( dirname( __FILE__ ).'/includes/class-form-controls.php' );

$kintone = new Kintone();
$kintone->register();

class Kintone {

	private $version = '';
	private $langs   = '';
	private $nonce   = 'kintone_';

	function __construct()
	{
		$data = get_file_data(
			__FILE__,
			array( 'ver' => 'Version', 'langs' => 'Domain Path' )
		);
		$this->version = $data['ver'];
		$this->langs   = $data['langs'];
	}

	public function register()
	{
		add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ) );
		add_action( 'template_redirect', array( $this, 'template_redirect' ) );
	}

	public function plugins_loaded()
	{
		load_plugin_textdomain(
			'kintone',
			false,
			dirname( plugin_basename( __FILE__ ) ).$this->langs
		);

		add_action( 'wp_enqueue_scripts', array( $this, 'wp_enqueue_scripts' ) );
		add_action( 'save_post', array( $this, 'save_post' ) );
		add_shortcode( 'kintone', array( $this, 'kintone' ) );
	}

	public function template_redirect()
	{
		if ( isset( $_POST['_kintone_nonce'] ) && $_POST['_kintone_nonce'] ) {
			if ( wp_verify_nonce( $_POST['_kintone_nonce'], $this->nonce.'-'.get_the_ID() ) ) {
				$result = Kintone_API::send( $_POST );
				if ( ! is_wp_error( $result ) ) {
					wp_safe_redirect( add_query_arg( array( 'saved' => true ), get_permalink( get_the_ID() ) ) );
					exit;
				}
			} else {
				$result = new WP_Error( 'error', __( 'Sorry, something unexpected happened.', 'kintone' ) );
			}
			update_option( 'kintone-error-message', $result->get_error_message() );
			update_option( 'kintone-error-data', $result->get_error_data() );
		}
	}

	public function kintone( $atts )
	{
		if ( ! is_singular() ) {
			return;
		}

		extract( shortcode_atts( array(
			'app'			 => '',
			'token'		   => '',
			'sub_domain'	  => '',
			'basic_auth_user' => '',
			'basic_auth_pass' => '',
			'orderby'		 => '',
			'order'		   => 'asc',
			'exclude'		 => '',
		), $atts ) );

		$option_key = Kintone_API::save_option_key( $atts );

		$meta_key = '_kintone-'.$sub_domain.'-'.$app;

		if ( is_user_logged_in() || !( $controls = get_post_meta( get_the_ID(), $meta_key, true ) ) ) {
			$controls = Kintone_API::get_form_json( $token, $app, $sub_domain, $basic_auth_user, $basic_auth_pass );
			if ( ! is_wp_error( $controls ) ) {
				update_post_meta( get_the_ID(), $meta_key, $controls, true );
			}
		} else {
			$controls = get_post_meta( get_the_ID(), $meta_key, true );
		}

		if ( is_wp_error( $controls ) ) {
			$html = '<div class="alert alert-danger">';
			$html .= '<h4 style="margin-bottom: 1em;">Kintone API Error</h4>';
			$html .= '<p>Error Message: '.$controls->get_error_message();
			$html .= '<br />Error Code: '.$controls->get_error_code().'</p>';
			$html .= '</div>';
		} else {
			$html = '<form class="kintone" role="form" method="post" enctype="multipart/form-data">'."\n";

			if ( get_option( 'kintone-error-message', false ) ) {
				$html .= '<div class="kintone-alert">' . get_option( 'kintone-error-message', false ) . '</div>';
				$error = get_option( 'kintone-error-data', array() );
				$html .= '<script type="application/javascript">var kintone_error = '.json_encode( $error ).';</script>';
				delete_option( 'kintone-error-message' );
				delete_option( 'kintone-error-data' );
			} elseif ( get_option( 'kintone-success', false ) ) {
				$html .= '<div class="kintone-alert kintone-alert-success">' . __( 'Your message was sent successfully.', 'kintone' ) . '</div>';
				delete_option( 'kintone-success' );
			}

			$html .= '<input type="hidden" name="_kintone_nonce" value="'.wp_create_nonce( $this->nonce.'-'.get_the_ID() ).'">';
			$html .= '<input type="hidden" name="_kintone_key" value="'.$option_key.'">';
			$controls = Kintone_API::exclude_controls( $controls, $exclude );
			$controls = Kintone_API::sort_controls( $controls, $order, $orderby );
			$html .= Kintone_API::parse_form( $controls )."\n";
			$html .= '<p><button type="submit" class="btn btn-primary btn-large">'.__( 'Submit', 'kintone' ).'</button></p>'."\n";
			$html .= "</form>";
		}

		add_action( 'wp_print_footer_scripts', array( $this, 'wp_print_footer_scripts' ), 9999 );

		return $html;
	}

	public function wp_enqueue_scripts()
	{
		wp_enqueue_style(
			'datetimepicker-style',
			plugins_url( 'lib/datetimepicker/jquery.datetimepicker.css', __FILE__ ),
			array(),
			$this->version,
			'all'
		);

		wp_enqueue_style(
			'kintone-style',
			plugins_url( 'css/kintone.css', __FILE__ ),
			array( 'datetimepicker-style' ),
			$this->version,
			'all'
		);

		wp_enqueue_script(
			'datetimepicker',
			plugins_url( 'lib/datetimepicker/jquery.datetimepicker.js', __FILE__ ),
			array( 'jquery' ),
			$this->version,
			true
		);

		wp_print_styles( 'editor-buttons' );
	}

	public function save_post( $post_id )
	{
		$keys = get_post_custom_keys( $post_id );
		if ( is_array( $keys ) ) {
			foreach ( $keys as $key ) {
				if ( preg_match( '/^_kintone/', $key ) ) {
					delete_post_meta( $post_id, $key );
				}
			}
		}
	}

	public function wp_print_footer_scripts()
	{
		global $month;
		$m = json_encode( array_values( $month ) );

		global $weekday_abbrev;
		$w = json_encode( array_values( $weekday_abbrev ) );

		$locale = substr( get_locale(), 0, 2 );
?>

<!-- Start Kintone -->
<script type="text/javascript">

(function($){

	if ( 'undefined' !== typeof kintone_error ) {
		for ( var e in kintone_error.errors) {
			var id = e.replace(/\]?.values?$/, '').replace(/^record(\.|\[)/, '');
			var message = kintone_error.errors[e].messages[0];
			if ($('#' + id)[0].nodeName == 'DIV') {
				$('label', '#' + id).addClass('kintone-alert-label');
				$('#' + id).append($('<div class="kintone-error-message" />').text(message));
			} else {
				$('#' + id).addClass('kintone-alert-input');
				$('#' + id).parent().append($('<div class="kintone-error-message" />').text(message));
			}
		}
	}

	$(".form-control.date").datetimepicker({
		lang:'<?php echo $locale; ?>',
		i18n:{
			<?php echo $locale; ?>: {
				months: <?php echo $m; ?>,
				dayOfWeek:  <?php echo $w; ?>
			},
		},
		scrollInput: false,
		timepicker: false,
		format: 'Y-m-d'
	});

	$(".form-control.time").datetimepicker({
		lang:'<?php echo $locale; ?>',
		i18n:{
			<?php echo $locale; ?>: {
				months: <?php echo $m; ?>,
				dayOfWeek:  <?php echo $w; ?>
			},
		},
		scrollInput: false,
		datepicker:false,
		format: 'H:i'
	});

	$(".form-control.datetime").datetimepicker({
		lang:'<?php echo $locale; ?>',
		i18n:{
			<?php echo $locale; ?>: {
				months: <?php echo $m; ?>,
				dayOfWeek:  <?php echo $w; ?>
			},
		},
		scrollInput: false,
		format: 'c'
	});

	Date.parseDate = function(input, format){
		return Date.parse(input);
	};

	Date.prototype.dateFormat = function(format){
		var offsetmin = this.getTimezoneOffset();
		var offset = 0 - (offsetmin / 60);
		if (Math.abs(offset) < 0) {
			offset = (10 > Math.abs(offset)) ? '-0' + Math.abs(offset) + ':00' : '-' + Math.abs(offset) + ':00';
		} else {
			offset = (10 > Math.abs(offset)) ? '+0' + Math.abs(offset) + ':00' : '+' + Math.abs(offset) + ':00';
		}

		var y = this.getFullYear();
		var m = (10 > (this.getMonth() + 1)) ? '0' + (this.getMonth() + 1) : this.getMonth() + 1;
		var d = (10 > this.getDate()) ? '0' + this.getDate() : this.getDate();
		var h = (10 > this.getHours()) ? '0' + this.getHours() : this.getHours();
		var i = (10 > this.getMinutes()) ? '0' + this.getMinutes() : this.getMinutes();

		switch( format ){
			case 'Y-m-d': return y + '-' + m + '-' + d;
			case 'H:i':   return h + ':' + i;
			case 'c':     return y + '-' + m + '-' + d + 'T' + h + ':' + i + offset;
		}
		// or default format
		return this.getDate() +'.'+(this.getMonth()+ 1)+'.'+this.getFullYear();
	};

})(jQuery);

</script>
<!-- End Kintone -->
<?php
	}
} // end class Kintone

// EOF
