<?php

final class Form_Controls
{
	/*
	* constructor
	*/
	private function __construct( )
	{

	}

	/*
	 * get instance
	 */
	public static function getInstance( )
	{
		/*
		 * a variable that keeps the sole instance.
		 */
		static $instance;

		if ( !isset( $instance ) ) {
			$instance = new Kintone_API( );
		}
		return $instance;
	}

	/*
	 * don't allow you to clone the instance.
	 */
	public final function __clone( ) {
		throw new RuntimeException( 'Clone is not allowed against ' );
	}

	public static function label( $args )
	{
		// $args = array(
		//	 "label" => "label_1",
		//	 "type" => "label",
		// );

		return sprintf(
			'<h3>%s</h3>',
			esc_html( $args['label'] )
		 );
	}

	public static function single_line_text( $args )
	{
		return self::generic_input_field( 'text', $args );
	}

	public static function number( $args )
	{
		return self::generic_input_field( 'number', $args );
	}

	public static function calc( $args )
	{
		return;
	}

	public static function multi_line_text( $args )
	{
		return self::generic_textarea_field( $args );
	}

	public static function check_box( $args )
	{
		return self::generic_checkbox_field( 'checkbox', $args );
	}

	public static function radio_button( $args )
	{
		$args['required'] = 'true'; // not true must be 'true'
		return self::generic_checkbox_field( 'radio', $args );
	}

	public static function drop_down( $args )
	{
		return self::generic_select_field( 'drop_down', $args );
	}

	public static function multi_select( $args )
	{
		return self::generic_select_field( 'multi_select', $args );
	}

	public static function file( $args )
	{
		//return self::generic_input_field( 'file', $args );
		return;
	}

	public static function date( $args )
	{
		return self::generic_input_field( 'text', $args );
	}

	public static function time( $args )
	{
		return self::generic_input_field( 'text', $args );
	}

	public static function datetime( $args )
	{
		return self::generic_input_field( 'text', $args );
	}

	public static function link( $args )
	{
		return self::generic_input_field( 'text', $args );
	}

	public static function spacer( $args )
	{
		return '<p>&nbsp;</p>';
	}

	public static function hr( $args )
	{
		return '<hr />';
	}


	public static function rich_text( $args )
	{
		require_once( ABSPATH . WPINC . '/class-wp-editor.php' );

		$settings = array(
			'wpautop'		  => true,
			'media_buttons'	=> false,
			'textarea_name'	=> $args['code'],
			'textarea_rows'	=> 10,
			'tabindex'		 => null,
			'editor_css'	   => '',
			'editor_class'	 => '',
			'teeny'			=> true,
			'dfw'			  => false,
			'tinymce'		  => true,
			'quicktags'		=> false,
			'drag_drop_upload' => false,
		 );

		$set = _WP_Editors::parse_settings( $args['code'], $settings );
		$editor_class = ' class="' . trim( $set['editor_class'] . ' wp-editor-area' ) . '"';
		$tabindex = $set['tabindex'] ? ' tabindex="' . ( int ) $set['tabindex'] . '"' : '';
		$switch_class = 'html-active';
		$toolbar = $buttons = $autocomplete = '';

		if ( $set['drag_drop_upload'] ) {
			_WP_Editors::$drag_drop_upload = true;
		}

		if ( ! empty( $set['editor_height'] ) )
			$height = ' style="height: ' . $set['editor_height'] . 'px"';
		else
			$height = ' rows="' . $set['textarea_rows'] . '"';

		$set['media_buttons'] = false;

		$wrap_class = 'wp-core-ui wp-editor-wrap ' . $switch_class;


		$editor = '<div id="wp-' . $args['code'] . '-wrap" class="' . $wrap_class . '">';

		/**
		 * Filter the HTML markup output that displays the editor.
		 *
		 * @since 2.1.0
		 *
		 * @param string $output Editor's HTML markup.
		 */
		$the_editor = apply_filters( 'the_editor', '<div id="wp-' . $args['code'] . '-editor-container" class="wp-editor-container">' .
			'<textarea' . $editor_class . $height . $tabindex . $autocomplete . ' cols="40" name="' . $set['textarea_name'] . '" ' .
			'id="' . $args['code'] . '">%s</textarea></div>' );

		$default = ( isset( $_POST[$args['code']] ) && $_POST[$args['code']] ) ? self::esc_html( $_POST[$args['code']] ) : self::esc_attr( $args['defaultValue'] );
		$editor .= sprintf( $the_editor, $default );
		$editor .=  "\n</div>\n\n";

		add_filter( 'teeny_mce_plugins', 'Form_Controls::teeny_mce_plugins' );

		_WP_Editors::editor_settings( $args['code'], $set );

		remove_filter( 'teeny_mce_plugins', 'Form_Controls::teeny_mce_plugins' );

		return sprintf(
			'<div class="form-group"><label><h4>%s</h4></label>%s</div>',
			$args['required'] === 'true' ? self::esc_html( $args['label'] ) . ' <span class="kintone-required">*</span>' : self::esc_html( $args['label'] ),
			$editor
		);
	}

	public static function teeny_mce_plugins($plugins)
	{
		return array( 'colorpicker', 'lists', 'image', 'wordpress', 'wpeditimage', 'wplink' );
	}

	/*
	 * Get an general input type field
	 * @param  string $type Value of type arg for input element, text or number
	 * @param  array  $args An hash array of control
	 * @return string	   html text
	 * @since  0.1
	 */
	protected static function generic_input_field( $type, $args )
	{
		// $args = array(
		//	 "code" => "string_1",
		//	 "defaultValue" => "",
		//	 "expression" => "",
		//	 "hideExpression" => "false",
		//	 "maxLength" => "64",
		//	 "minLength" => null,
		//	 "label" => "string_1",
		//	 "noLabel" => "false",
		//	 "required" => "true",
		//	 "type" => "SINGLE_LINE_TEXT",
		//	 "unique" => "true",
		// );

		$args = self::set_args( $args );

		$control = sprintf(
			'<input type="%1$s" class="form-control %2$s" id="%3$s" name="%3$s" data-field_type="%4$s" data-expression="%5$s" value="%6$s" maxlength="%7$s" min="%8$s" placeholder="%9$s" %10$s />',
			$type,
			self::esc_attr( strtolower( $args['type'] ) ),
			self::esc_attr( $args['code'] ),
			self::esc_attr( $args['type'] ),
			self::esc_attr( $args['expression'] ),
			( isset( $_POST[$args['code']] ) && $_POST[$args['code']] ) ? esc_html( $_POST[$args['code']] ) : self::esc_attr( $args['defaultValue'] ),
			self::esc_attr( $args['maxLength'] ),
			self::esc_attr( $args['minLength'] ),
			$args['hideExpression'] == 'true' ? '' : self::esc_attr( $args['expression'] ),
			$args['required'] === 'true' ? 'required="required"' : ''
		 );

		$html = '<div class="form-group">';
		if ( $args['noLabel'] !== 'true' ) {
			$html .= sprintf(
				'<label><h4>%s</h4>%s</label>',
				$args['required'] === 'true' ? self::esc_html( $args['label'] ) . ' <span class="kintone-required">*</span>' : self::esc_html( $args['label'] ),
				$control
			 );
		} else {
			$html .= $control;
		}
		$html .= '</div>';

		return $html;
	}


	/*
	 * Get an general textarea field
	 * @param  array  $args An hash array of control
	 * @return string	   html text
	 * @since  0.1
	 */
	protected static function generic_textarea_field( $args )
	{
		// $args = array(
		//	 "code" => "string_1",
		//	 "defaultValue" => "this is text",
		//	 "label" => "string_1",
		//	 "noLabel" => "false",
		//	 "required" => "false",
		//	 "type" => "RICH_TEXT",
		// );

		$args = self::set_args( $args );

		$control = sprintf(
			'<textarea class="form-control %1$s" id="%2$s" name="%2$s" data-field_type="%3$s" %4$s>%5$s</textarea>',
			self::esc_attr( strtolower( $args['type'] ) ),
			self::esc_attr( $args['code'] ),
			self::esc_attr( $args['type'] ),
			$args['required'] === 'true' ? 'required="required"' : '',
			( isset( $_POST[$args['code']] ) && $_POST[$args['code']] ) ? esc_html( $_POST[$args['code']] ) : self::esc_textarea( $args['defaultValue'] )
		 );

		$html = '<div class="form-group">';
		if ( $args['noLabel'] !== 'true' ) {
			$html .= sprintf(
				'<label><h4>%s</h4>%s</label>',
				$args['required'] === 'true' ? self::esc_html( $args['label'] ) . ' <span class="kintone-required">*</span>' : self::esc_html( $args['label'] ),
				$control
			 );
		} else {
			$html .= $control;
		}
		$html .= '</div>';

		return $html;
	}


	/*
	 * Get an general input type field
	 * @param  string $type Value of type arg for input element, checkbox or radio
	 * @param  array  $args An hash array of control
	 * @return string	   html text
	 * @since  0.1
	 */
	protected static function generic_checkbox_field( $type, $args )
	{
		// $args = array(
		//	 "code" => "string_1",
		//	 "defaultValue" => "this is text",
		//	 "label" => "string_1",
		//	 "noLabel" => "false",
		//	 "required" => "false",
		//	 "type" => "CHECK_BOX",
		//	 "options" => array(
		//		 "sample1",
		//		 "sample2",
		//		 "sample3"
		//	 ),
		// );

		$args = self::set_args( $args );

		$control = null;
		foreach ( $args['options'] as $option ) {
			$control .= sprintf(
				'<div class="%s">',
				$type
			 );

			$default = '';
			if ( isset($_POST[$args['code']]) && is_array($_POST[$args['code']]) && in_array( $option, $_POST[$args['code']] ) ) {
				$default = 'checked="checked"';
			} elseif ( $args['defaultValue'] === $option ) {
				$default = 'checked="checked"';
			}

			$control .= sprintf(
				'<label><input type="%s" name="%s" class="form-control" value="%s" %s />%s</label>',
				$type,
				$type === 'checkbox' ? self::esc_attr( $args['code'] ).'[]' : self::esc_attr( $args['code'] ),
				self::esc_attr( $option ),
				$default,
				self::esc_html( $option )
			 );
			$control .= '</div>';
		}

		$html = sprintf(
			'<div class="form-group %s" id="%s" data-field_type="%s" data-required="%s">',
			self::esc_attr( strtolower( $args['type'] ) ),
			self::esc_attr( $args['code'] ),
			self::esc_attr( $args['type'] ),
			$args['required'] === 'true' ? 'true' : 'false'
		 );
		if ( $args['noLabel'] !== 'true' ) {
			$html .= sprintf(
				'<h4>%s</h4>%s',
				$args['required'] === 'true' ? self::esc_html( $args['label'] ) . ' <span class="kintone-required">*</span>' : self::esc_html( $args['label'] ),
				$control
			 );
		} else {
			$html .= $control;
		}
		$html .= '</div>';
		return $html;
	}


	/*
	 * Get an general select type field
	 * @param  string $type Value of type arg for input element, checkbox or radio
	 * @param  array  $args An hash array of control
	 * @return string	   html text
	 * @since  0.1
	 */
	protected static function generic_select_field( $type, $args )
	{
		// $args = array(
		//	 "code" => "string_1",
		//	 "defaultValue" => "this is text",
		//	 "label" => "string_1",
		//	 "noLabel" => "false",
		//	 "required" => "false",
		//	 "type" => "MULTI_SELECT",
		//	 "options" => array(
		//		 "sample1",
		//		 "sample2",
		//		 "sample3"
		//	 ),
		// );

		$args = self::set_args( $args );

		$control = sprintf(
			'<select class="form-control %1$s" id="%2$s" name="%2$s" data-field_type="%3$s" %4$s %5$s>',
			self::esc_attr( strtolower( $args['type'] ) ),
			self::esc_attr( $args['code'] ),
			self::esc_attr( $args['type'] ),
			$args['required'] === 'true' ? 'required="required"' : '',
			$type === 'multi_select' ? 'multiple="multiple" size="5"' : ''
		 );

		foreach ( $args['options'] as $option ) {
			$default = '';
			if ( isset($_POST[$args['code']]) && is_array($_POST[$args['code']]) && in_array( $option, $_POST[$args['code']] ) ) {
				$default = 'selected="selected"';
			} elseif ( $args['defaultValue'] === $option ) {
				$default = 'selected="selected"';
			}

			$control .= sprintf(
				'<option %s>%s</option>',
				$default,
				self::esc_html( $option )
			 );
		}

		$control .= '</select>';

		$html = '<div class="form-group">';
		if ( $args['noLabel'] !== 'true' ) {
			$html .= sprintf(
				'<label><h4>%s</h4>%s</label>',
				$args['required'] === 'true' ? self::esc_html( $args['label'] ) . ' <span class="kintone-required">*</span>' : self::esc_html( $args['label'] ),
				$control
			 );
		} else {
			$html .= $control;
		}
		$html .= '</div>';

		return $html;
	}


	protected static function esc_html( $str )
	{
		return esc_html( str_replace( '%', '%%', $str ) );
	}

	protected static function esc_textarea( $str )
	{
		return esc_textarea( str_replace( '%', '%%', $str ) );
	}

	protected static function esc_attr( $str )
	{
		return esc_attr( str_replace( '%', '%%', $str ) );
	}

	protected static function set_args( $args )
	{
		$keys = array(
			"label",
			"code",
			"elementId",
			"noLabel",
			"required",
			"unique",
			"maxValue",
			"minValue",
			"maxLength",
			"minLength",
			"defaultValue",
			"defaultExpression",
			"options",
			"expression",
			"digit",
			"protocol",
			"format",
			"displayScale",
			"hideExpression",
		 );

		foreach ( $keys as $key ) {
			if ( !isset( $args[$key] ) ) {
				$args[$key] = '';
			}
		}

		return $args;
	}

}


// EOF
